@@ -25,6 +25,9 @@ our $PROFILES_DB = {};    # Class-wide, caches all profiles that get used while
 our $is_bibtex  = qr/(^literal\:\s*\@)|(\.bib$)/;
 our $is_archive = qr/(^literal\:PK)|(\.zip$)/;
 
+use base qw(Exporter);
+our @EXPORT = (qw(addMathFormat removeMathFormat maybeAddMathFormat));
+
 sub new {
   my ($class, %opts) = @_;
   #TODO: How about defaults in the daemon server use case? Should we support those here?
@@ -67,7 +70,7 @@ sub getopt_specification {
     "mode=s"      => \$$opts{profile},
     "source=s"    => \$$opts{source},
     # Output framing
-    "embed" => sub { $$opts{whatsout} = 'fragment'; },
+    "embed"      => sub { $$opts{whatsout} = 'fragment'; },
     "whatsin=s"  => \$$opts{whatsin},
     "whatsout=s" => \$$opts{whatsout},
     # Daemon options
@@ -93,23 +96,21 @@ sub getopt_specification {
     "linelength=i"                => \$$opts{linelength},
     "plane1!"                     => \$$opts{plane1},
     "hackplane1!"                 => \$$opts{hackplane1},
-    "mathimages"                  => sub { _addMathFormat($opts, 'images'); },
-    "nomathimages"                => sub { _removeMathFormat($opts, 'images'); },
-    "mathsvg"                     => sub { _addMathFormat($opts, 'svg'); },
-    "nomathsvg"                   => sub { _removeMathFormat($opts, 'svg'); },
-    "presentationmathml|pmml"     => sub { _addMathFormat($opts, 'pmml'); },
-    "contentmathml|cmml"          => sub { _addMathFormat($opts, 'cmml'); },
-    "openmath|om"                 => sub { _addMathFormat($opts, 'om'); },
-    "keepXMath|xmath"             => sub { _addMathFormat($opts, 'xmath'); },
-    "nopresentationmathml|nopmml" => sub { _removeMathFormat($opts, 'pmml'); },
-    "nocontentmathml|nocmml"      => sub { _removeMathFormat($opts, 'cmml'); },
-    "noopenmath|noom"             => sub { _removeMathFormat($opts, 'om'); },
-    "nokeepXMath|noxmath"         => sub { _removeMathFormat($opts, 'xmath'); },
-    "mathtex"                     => sub { _addMathFormat($opts, 'mathtex'); },
-    "nomathtex"                   => sub { _removeMathFormat($opts, 'mathtex'); },
-    "mathlex"                     => sub { _addMathFormat($opts, 'mathlex'); },
-    "nomathlex"                   => sub { _removeMathFormat($opts, 'mathlex'); },
-    "parallelmath!"                => \$$opts{parallelmath},
+    "mathimages"                  => sub { addMathFormat($opts, 'images'); },
+    "nomathimages"                => sub { removeMathFormat($opts, 'images'); },
+    "mathsvg"                     => sub { addMathFormat($opts, 'svg'); },
+    "nomathsvg"                   => sub { removeMathFormat($opts, 'svg'); },
+    "presentationmathml|pmml"     => sub { addMathFormat($opts, 'pmml'); },
+    "contentmathml|cmml"          => sub { addMathFormat($opts, 'cmml'); },
+    "openmath|om"                 => sub { addMathFormat($opts, 'om'); },
+    "keepXMath|xmath"             => sub { addMathFormat($opts, 'xmath'); },
+    "nopresentationmathml|nopmml" => sub { removeMathFormat($opts, 'pmml'); },
+    "nocontentmathml|nocmml"      => sub { removeMathFormat($opts, 'cmml'); },
+    "noopenmath|noom"             => sub { removeMathFormat($opts, 'om'); },
+    "nokeepXMath|noxmath"         => sub { removeMathFormat($opts, 'xmath'); },
+    "mathtex"                     => sub { addMathFormat($opts, 'mathtex'); },
+    "nomathtex"                   => sub { removeMathFormat($opts, 'mathtex'); },
+    "parallelmath!"               => \$$opts{parallelmath},
     # Some general XSLT/CSS/JavaScript options.
     "stylesheet=s"      => \$$opts{stylesheet},
     "xsltparameter=s"   => \@{ $$opts{xsltparameters} },
@@ -118,7 +119,7 @@ sub getopt_specification {
     "javascript=s"      => \@{ $$opts{javascript} },
     "icon=s"            => \$$opts{icon},
     # Options for broader document set processing
-    "split!" => \$$opts{split},
+    "split!"    => \$$opts{split},
     "splitat=s" => sub { $$opts{splitat} = $_[1];
       $$opts{split} = 1 unless defined $$opts{split}; },
     "splitpath=s" => sub { $$opts{splitpath} = $_[1];
@@ -158,15 +159,15 @@ sub getopt_specification {
   };
   return ($spec, $opts) unless ($options{type} && ($options{type} eq 'keyvals'));
   # Representation use case:
-  my $keyvals = $options{keyvals} || [];
-  my $rep_spec = {};    # Representation specification
+  my $keyvals  = $options{keyvals} || [];
+  my $rep_spec = {};                        # Representation specification
   foreach my $key (keys %$spec) {
     if ($key =~ /^(.+)=\w$/) {
       my $name = $1;
       $$rep_spec{$key} = sub { CORE::push @$keyvals, [$name, $_[1]] };
     } else {
       $$rep_spec{$key} = sub {
-        my $ctl = $_[0]->{ctl};
+        my $ctl  = $_[0]->{ctl};
         my $used = ($$ctl[0] ? 'no' : '') . $$ctl[1];
         CORE::push @$keyvals, [$used, undef] };
     }
@@ -182,16 +183,16 @@ sub read {
   my $opts = $$self{opts};
   local @ARGV = @$argref;
   my ($spec) = getopt_specification(options => $opts);
-  my $silent = %read_options && $read_options{silent};
+  my $silent             = %read_options && $read_options{silent};
   my $getOptions_success = GetOptions(%{$spec});
   if (!$getOptions_success && !$silent) {
     pod2usage(-message => $LaTeXML::IDENTITY, -exitval => 1, -verbose => 99,
-      -input => pod_where({ -inc => 1 }, __PACKAGE__),
+      -input    => pod_where({ -inc => 1 }, __PACKAGE__),
       -sections => 'OPTION SYNOPSIS', -output => \*STDERR);
   }
   if (!$silent && $$opts{help}) {
-    pod2usage(-message => $LaTeXML::IDENTITY, -exitval => 1, -verbose => 99,
-      -input => pod_where({ -inc => 1 }, __PACKAGE__),
+    pod2usage(-message => $LaTeXML::IDENTITY, -exitval => 0, -verbose => 99,
+      -input    => pod_where({ -inc => 1 }, __PACKAGE__),
       -sections => 'OPTION SYNOPSIS', output => \*STDOUT);
   }
 
@@ -204,7 +205,7 @@ sub read {
   # Removed math formats are irrelevant for conversion:
   delete $$opts{removed_math_formats};
 
-  if ($$opts{showversion}) { print STDERR "$LaTeXML::IDENTITY\n"; exit(1); }
+  if ($$opts{showversion}) { print STDERR "$LaTeXML::IDENTITY\n"; exit(0); }
 
   $$opts{source} = $ARGV[0] unless $$opts{source};
   # Special source-based guessing needs to happen here,
@@ -233,11 +234,11 @@ sub scan_to_keyvals {
   my ($self, $argref, %read_options) = @_;
   local @ARGV = @$argref;
   my ($spec, $keyvals) = getopt_specification(type => 'keyvals');
-  my $silent = %read_options && $read_options{silent};
+  my $silent             = %read_options && $read_options{silent};
   my $getOptions_success = GetOptions(%$spec);
   if (!$getOptions_success && !$silent) {
     pod2usage(-message => $LaTeXML::IDENTITY, -exitval => 1, -verbose => 99,
-      -input => pod_where({ -inc => 1 }, __PACKAGE__),
+      -input    => pod_where({ -inc => 1 }, __PACKAGE__),
       -sections => 'OPTION SYNOPSIS', -output => \*STDERR);
   }
   CORE::push @$keyvals, ['source', $ARGV[0]] if $ARGV[0];
@@ -308,14 +309,14 @@ sub check {
 sub _obey_profile {
   my ($self) = @_;
   $$self{dirty} = 1;
-  my $opts = $$self{opts};
+  my $opts    = $$self{opts};
   my $profile = lc($$opts{profile} || 'custom');
   $profile =~ s/\.opt$//;
   # Look at the PROFILES_DB or find a profiles file (otherwise fallback to custom)
   my $profile_opts = {};
   if ($profile ne 'custom') {
     if (defined $$PROFILES_DB{$profile}) {
-      %$profile_opts = %{ $$PROFILES_DB{$profile} }
+      %$profile_opts = %{ $$PROFILES_DB{$profile} };
     } elsif (my $file = pathname_find($profile . '.opt', paths => $$opts{paths},
         types => [], installation_subdir => 'resources/Profiles')) {
       my $conf_tmp = LaTeXML::Common::Config->new;
@@ -375,6 +376,7 @@ sub _prepare_options {
   $$opts{timeout}     = 600          unless defined $$opts{timeout};       # 10 minute timeout default
   $$opts{expire}      = 600          unless defined $$opts{expire};        # 10 minute timeout default
   $$opts{mathparse}   = 'RecDescent' unless defined $$opts{mathparse};
+  $$opts{inputencoding} = "utf-8" unless defined $$opts{inputencoding};
   if ($$opts{mathparse} eq 'no') {
     $$opts{mathparse}   = 0;
     $$opts{nomathparse} = 1; }                                             #Backwards compatible
@@ -413,7 +415,7 @@ sub _prepare_options {
       $$opts{whatsout} = 'archive';
     } else {
       $$opts{whatsout} = 'document';
-    } }
+  } }
   if ($$opts{format}) {
     # Lower-case for sanity's sake
     $$opts{format} = lc($$opts{format});
@@ -485,19 +487,18 @@ sub _prepare_options {
     if ($$opts{split}) {
       $$opts{splitat}     = 'section' unless defined $$opts{splitat};
       $$opts{splitnaming} = 'id'      unless defined $$opts{splitnaming};
-      $$opts{splitback} = "//ltx:bibliography | //ltx:appendix | //ltx:index" unless defined $$opts{splitback};
-      $$opts{splitpaths} = {
-        part    => "//ltx:part | " . $$opts{splitback},
-        chapter => "//ltx:part | //ltx:chapter | " . $$opts{splitback},
-        section => "//ltx:part | //ltx:chapter | //ltx:section | " . $$opts{splitback},
-        subsection => "//ltx:part | //ltx:chapter | //ltx:section | //ltx:subsection | " . $$opts{splitback},
-        subsubsection => "//ltx:part | //ltx:chapter | //ltx:section | //ltx:subsection | //ltx:subsubsection | " . $$opts{splitback} }
-        unless defined $$opts{splitpaths};
+      $$opts{splitancestors} = {
+        part          => [qw()],
+        chapter       => [qw(part)],
+        section       => [qw(part chapter)],
+        subsection    => [qw(part chapter section)],
+        subsubsection => [qw(part chapter section subsection)] };
+      $$opts{splitback} = [qw(bibliography appendix index)];
 
       $$opts{splitnaming} = _checkOptionValue('--splitnaming', $$opts{splitnaming},
         qw(id idrelative label labelrelative));
-      $$opts{splitat} = _checkOptionValue('--splitat', $$opts{splitat}, CORE::keys %{ $$opts{splitpaths} });
-      $$opts{splitpath} = $$opts{splitpaths}->{ $$opts{splitat} } unless defined $$opts{splitpath}; }
+      $$opts{splitat} = _checkOptionValue('--splitat', $$opts{splitat}, CORE::keys %{ $$opts{splitancestors} });
+      $$opts{splitpath} = make_splitpaths($opts, $$opts{splitat}) unless defined $$opts{splitpath}; }
     # Check for appropriate combination of split, scan, prescan, dbfile, crossref
     if ($$opts{split} && (!defined $$opts{destination}) && ($$opts{whatsout} !~ /^archive/)) {
       croak("Must supply --destination when using --split"); }
@@ -547,7 +548,7 @@ sub _prepare_options {
         && scalar(grep { $_ ne 'images' } @{ $$opts{math_formats} });
       croak("Default html stylesheet does not support svg") if $$opts{svg};
       $$opts{math_formats} = [];
-      _maybeAddMathFormat($opts, 'images');
+      maybeAddMathFormat($opts, 'images');
     }
     $$opts{svg} = 1 unless defined $$opts{svg};      # If we're not making HTML, SVG is on by default
           # PMML default if we're HTMLy and all else fails and no mathimages:
@@ -560,34 +561,37 @@ sub _prepare_options {
   }
   # If really nothing hints to define format, then default it to XML
   $$opts{format} = 'xml' unless defined $$opts{format};
-  $$self{dirty} = 0;
+  $$self{dirty}  = 0;
   return; }
 
-## Utilities:
+## Public Utilities:
 
-sub _addMathFormat {
+sub addMathFormat {
   my ($opts, $fmt) = @_;
   $$opts{math_formats} = [] unless defined $$opts{math_formats};
   CORE::push(@{ $$opts{math_formats} }, $fmt)
     unless (grep { $_ eq $fmt } @{ $$opts{math_formats} }) || $$opts{removed_math_formats}->{$fmt};
   return; }
 
-sub _removeMathFormat {
+sub removeMathFormat {
   my ($opts, $fmt) = @_;
   @{ $$opts{math_formats} } = grep { $_ ne $fmt } @{ $$opts{math_formats} };
   $$opts{removed_math_formats}->{$fmt} = 1;
   return; }
 
-sub _maybeAddMathFormat {
+# Add a default math format, when no math formatter is requested, unless specifically forbidden
+sub maybeAddMathFormat {
   my ($opts, $fmt) = @_;
   unshift(@{ $$opts{math_formats} }, $fmt)
-    unless (grep { $_ eq $fmt } @{ $$opts{math_formats} }) || $$opts{removed_math_formats}{$fmt};
+    unless @{ $$opts{math_formats} } || $$opts{removed_math_formats}{$fmt};
   return; }
 
 sub _checkMathFormat {
   my ($opts, $fmt) = @_;
   return grep { $_ eq $fmt } @{ $$opts{math_formats} }; }
 
+## Utilities:
+
 sub _checkOptionValue {
   my ($option, $value, @choices) = @_;
   if ($value) {
@@ -595,6 +599,20 @@ sub _checkOptionValue {
       return $choice if substr($choice, 0, length($value)) eq $value; } }
   croak("Value for $option, $value, doesn't match " . join(', ', @choices)); }
 
+# Contrived xpath, since backmatter can now be at any level!
+sub make_splitpaths {
+  my ($opts, $splitat) = @_;
+  my @paths = ();
+  my $anc   = $$opts{splitancestors}{$splitat};
+  foreach my $unit ($splitat, ($anc ? @$anc : ())) {
+    CORE::push(@paths, "//ltx:$unit");
+    foreach my $back (@{ $$opts{splitback} }) {
+      CORE::push(@paths, "//ltx:$back\["
+          . join(' or ', "preceding-sibling::ltx:$unit",
+          map { "parent::ltx:$_"; } @{ $$opts{splitancestors}{$unit} })
+          . "]"); } }
+  return join(' | ', @paths); }
+
 ### This is from t/lib/TestDaemon.pm and ideally belongs in Util::Pathname
 sub _read_options_file {
   my ($file) = @_;
@@ -881,9 +899,6 @@ latexmlc [options]
                          representation (default is to remove)
  --mathtex               adds TeX annotation to parallel markup
  --nomathtex             disables the above (default)
- --mathlex               (EXPERIMENTAL) adds linguistic lexeme 
-                         annotation to parallel markup
- --nomathlex             (EXPERIMENTAL) disables the above (default)
  --parallelmath          use parallel math annotations (default)
  --noparallelmath        disable parallel math annotations
  --plane1                use plane-1 unicode for symbols
@@ -1121,7 +1136,7 @@ Variety of shorthand profiles.
     You can examine any of them in their C<resources/Profiles/name.opt>
     file.
 
-Example: C<latexmlc --profile=math '1+2=3'>
+Example: C<latexmlc --profile=math 'literal:1+2=3'>
 
 =item C<--omitdoctype>, C<--noomitdoctype>
 
