@@ -40,6 +40,7 @@ sub rewrite {
         $$self{labels}{$label} = $id; } }
     else {
       Error('malformed', 'label', $node, "Node has labels but no xml:id"); } }
+  print STDERR "\n" . ('=' x 40) . "\n" if $LaTeXML::Core::Rewrite::DEBUG;
   $self->applyClause($document, $node, 0, $self->clauses);
   return; }
 
@@ -73,55 +74,46 @@ sub getLabelID {
 #   regexp  => $string: apply regexp (subst) to all text nodes in/under the current node.
 
 sub applyClause {
-  my ($self, $document, $tree, $n_to_replace, $clause, @more_clauses) = @_;
+  my ($self, $document, $tree, $nmatched, $clause, @more_clauses) = @_;
+  if (!$clause) {
+    markSeen($tree, $nmatched);
+    return; }
+  return unless $clause;
   if ($$clause[0] eq 'uncompiled') {
     $self->compileClause($document, $clause); }
   my ($ignore, $op, $pattern) = @$clause;
   if ($op eq 'trace') {
     local $LaTeXML::Core::Rewrite::DEBUG = 1;
-    $self->applyClause($document, $tree, $n_to_replace, @more_clauses); }
+    $self->applyClause($document, $tree, $nmatched, @more_clauses); }
   elsif ($op eq 'ignore') {
-    $self->applyClause($document, $tree, $n_to_replace, @more_clauses); }
+    $self->applyClause($document, $tree, $nmatched, @more_clauses); }
   elsif ($op eq 'select') {
-    my ($xpath, $nnodes) = @$pattern;
+    my ($xpath, $nnodes, @wilds) = @$pattern;
     my @matches = $document->findnodes($xpath, $tree);
     print STDERR "Rewrite selecting \"$xpath\" => " . scalar(@matches) . " matches\n"
       if $LaTeXML::Core::Rewrite::DEBUG;
     foreach my $node (@matches) {
       next unless $node->ownerDocument->isSameNode($tree->ownerDocument); # If still attached to original document!
-      $self->applyClause($document, $node, $nnodes, @more_clauses); } }
+      next if $node->getAttribute('_matched');
+      my @w = markWildcards($node, @wilds);
+      $self->applyClause($document, $node, $nnodes, @more_clauses);
+      unmarkWildcards($node, @w); } }
   elsif ($op eq 'multi_select') {
     foreach my $subpattern (@$pattern) {
-      my ($xpath, $nnodes) = @$subpattern;
+      my ($xpath, $nnodes, @wilds) = @$subpattern;
       my @matches = $document->findnodes($xpath, $tree);
       print STDERR "Rewrite selecting \"$xpath\" => " . scalar(@matches) . " matches\n"
         if $LaTeXML::Core::Rewrite::DEBUG;
       foreach my $node (@matches) {
         next unless $node->ownerDocument->isSameNode($tree->ownerDocument); # If still attached to original document!
-        $self->applyClause($document, $node, $nnodes, @more_clauses); } } }
+        my @w = markWildcards($node, @wilds);
+        $self->applyClause($document, $node, $nnodes, @more_clauses);
+        unmarkWildcards($node, @w); } } }
   elsif ($op eq 'test') {
     my $nnodes = &$pattern($document, $tree);
     print STDERR "Rewrite test at " . $tree->toString . ": " . ($nnodes ? $nnodes . " to replace" : "failed") . "\n"
       if $LaTeXML::Core::Rewrite::DEBUG;
     $self->applyClause($document, $tree, $nnodes, @more_clauses) if $nnodes; }
-  elsif ($op eq 'wrap') {
-    if ($n_to_replace > 1) {
-      my $parent = $tree->parentNode;
-      # Remove & separate nodes to be replaced, and sibling nodes following them.
-      my @following = ();    # Collect the matching and following nodes
-      while (my $sib = $parent->lastChild) {
-        $parent->removeChild($sib);
-        unshift(@following, $sib);
-        last if $$sib == $$tree; }
-      my @replaced = map { shift(@following) } 1 .. $n_to_replace;    # Remove the nodes to be replaced
-      $document->setNode($parent);
-      $tree = $document->openElement('ltx:XMWrap', font => $document->getNodeFont($parent));
-      print STDERR "Wrapping " . join(' ', map { Stringify($_) } @replaced) . "\n"
-        if $LaTeXML::Core::Rewrite::DEBUG;
-      map { $tree->appendChild($_) } @replaced;                       # Add matched nodes to XMWrap
-      map { $parent->appendChild($_) } @following;                    # Add back the following nodes.
-    }
-    $self->applyClause($document, $tree, 1, @more_clauses); }
   elsif ($op eq 'replace') {
     print STDERR "Rewrite replace at " . $tree->toString . " using $pattern\n"
       if $LaTeXML::Core::Rewrite::DEBUG;
@@ -132,15 +124,15 @@ sub applyClause {
       $parent->removeChild($sib);
       unshift(@following, $sib);
       last if $tree->isSameNode($sib); }
-    my @replaced = map { shift(@following) } 1 .. $n_to_replace;    # Remove the nodes to be replaced
+    my @replaced = map { shift(@following) } 1 .. $nmatched;    # Remove the nodes to be replaced
     map { $document->unRecordNodeIDs($_) } @replaced;
     # Carry out the operation, inserting whatever nodes.
     $document->setNode($parent);
     my $point = $parent->lastChild;
-    &$pattern($document, @replaced);                                # Carry out the insertion.
+    &$pattern($document, @replaced);                            # Carry out the insertion.
 
     # Now collect the newly inserted nodes for any needed patching
-    my @inserted = ();                                              # Collect the newly added nodes.
+    my @inserted = ();                                          # Collect the newly added nodes.
     if ($point) {
       my @sibs = $parent->childNodes;
       while (my $sib = pop(@sibs)) {
@@ -151,7 +143,7 @@ sub applyClause {
 
     # Now make any adjustments to the new nodes
     map { $document->recordNodeIDs($_) } @inserted;
-    my $font = $document->getNodeFont($tree);                       # the font of the matched node
+    my $font = $document->getNodeFont($tree);                   # the font of the matched node
     foreach my $ins (@inserted) {    # Copy the non-semantic parts of font to the replacement
       $document->mergeNodeFontRec($ins => $font); }
     # Now, replace the following nodes.
@@ -161,10 +153,19 @@ sub applyClause {
       if $LaTeXML::Core::Rewrite::DEBUG;
     &$pattern($tree); }
   elsif ($op eq 'attributes') {
-    map { $tree->setAttribute($_, $$pattern{$_}) } keys %$pattern;
-    print STDERR "Rewrite attributes for " . Stringify($tree) . "\n"
+    my @nodes = ();
+    my $n     = $tree;
+    for (my $i = 0 ; $n && ($i < $nmatched) ; $i++) {
+      push(@nodes, $n);
+      $n = $n->nextSibling; }
+    if ($tree->hasAttribute('_has_wildcards')) {
+      setAttributes_wild($document, $pattern, @nodes); }
+    else {
+      setAttributes_encapsulate($document, $pattern, @nodes); }
+
+    print STDERR "Rewrite attributes (deep $nmatched) " . join(',', sort keys %$pattern) . " for " . Stringify($tree) . "\n"
       if $LaTeXML::Core::Rewrite::DEBUG;
-  }
+    $self->applyClause($document, $tree, $nmatched, @more_clauses); }
   elsif ($op eq 'regexp') {
     my @matches = $document->findnodes('descendant-or-self::text()', $tree);
     print STDERR "Rewrite regexp => " . scalar(@matches) . " matches\n"
@@ -177,6 +178,106 @@ sub applyClause {
     Error('misdefined', '<rewrite>', undef, "Unknown directive '$op' in Compiled Rewrite spec"); }
   return; }
 
+# Set attributes for an encapsulated tree (ie. a decorated symbol as symbol itself)
+sub setAttributes_encapsulate {
+  my ($document, $attributes, @nodes) = @_;
+  return unless grep { !$_->getAttribute('_matched'); } @nodes;
+  my $node = $nodes[0];
+  if (!$$attributes{_nowrap} && (scalar(@nodes) > 1)) {
+    $node = $document->wrapNodes('ltx:XMWrap', @nodes); }
+  map { $node->setAttribute($_, $$attributes{$_}) } keys %$attributes;    #}
+  return; }
+
+# Set attributes for a subtree w/wildcards
+# Presumably only on tokens which are not in the wildcard?
+sub setAttributes_wild {
+  my ($document, $attributes, @nodes) = @_;
+  my $node = $nodes[0];
+  return unless grep { !$_->getAttribute('_matched'); } @nodes;
+  if ($$attributes{_nowrap}    # No wrapping requested, or already is an XMDual
+    || ((scalar(@nodes) == 1) && ($document->getNodeQName($nodes[0]) eq 'ltx:XMDual'))) {
+    my ($nonwild) = grep { !$_->getAttribute('_wildcard'); } @nodes;
+    if ($nonwild) {
+      map { $nonwild->setAttribute($_ => $$attributes{$_}); } keys %$attributes; } }
+  else {
+    # Do this slightly clunky, in order to keep the SAME xml @nodes in the result
+    my $wrapper = $document->wrapNodes('ltx:XMWrap', @nodes);
+    my @wildids = set_wildcard_ids($document, $wrapper);
+    $node = $document->wrapNodes('ltx:XMDual', $wrapper);
+    $node->setAttribute(role => $$attributes{role}) if defined $$attributes{role};
+    $node->removeChild($wrapper);
+    my $app = $document->openElementAt($node, 'ltx:XMApp');
+    my $op  = $document->openElementAt($app, 'ltx:XMTok',
+      map { ($_ eq 'role' ? () : ($_ => $$attributes{$_})); } keys %$attributes);
+    foreach my $rid (@wildids) {
+      $document->openElementAt($app, 'ltx:XMRef', idref => $rid); }
+    $node->appendChild($wrapper); }
+  return; }
+
+sub set_wildcard_ids {
+  my ($document, $node) = @_;
+  if (($node->nodeType != XML_ELEMENT_NODE)
+    || $node->getAttribute('_matched')) {
+    return (); }
+  elsif ($node->hasAttribute('_wildcard')) {
+    my $id = $node->getAttribute('xml:id');
+    if (!$id) {
+      LaTeXML::Package::GenerateID($document, $node, undef, '');
+      $id = $node->getAttribute('xml:id'); }
+    return ($id); }
+  else {
+    return map { set_wildcard_ids($document, $_); } $node->childNodes; } }
+
+sub markSeen {
+  my ($node, $nsibs) = @_;
+  for (my $i = 0 ; $node && ($i < $nsibs) ; $i++) {
+    markSeen_rec($node);
+    $node = $node->nextSibling; }
+  return; }
+
+sub markSeen_rec {
+  my ($node) = @_;
+  return if $node->getAttribute('_wildcard');    # Not even children?
+  $node->setAttribute('_matched' => 1);
+  foreach my $child ($node->childNodes) {
+    if ($child->nodeType == XML_ELEMENT_NODE) {
+      markSeen_rec($child); } }
+  return; }
+
+sub markWildcards {
+  my ($node, @wilds) = @_;
+  $node->setAttribute(_has_wildcards => 1) if @wilds;
+  my @n = ();
+  foreach my $wild (@wilds) {
+    my $n     = $node;
+    my $start = 1;
+    foreach my $i (@$wild) {
+      last unless $n;
+      $n     = ($start ? nth_sibling($n, $i) : nth_child($n, $i));
+      $start = 0; }
+    if ($n && ($n->nodeType == XML_ELEMENT_NODE)) {
+      $n->setAttribute('_wildcard' => 1);
+      push(@n, $n); } }
+  return @n; }
+
+sub unmarkWildcards {
+  my (@nodes) = @_;
+  foreach my $n (@nodes) {
+    if ($n && ($n->nodeType == XML_ELEMENT_NODE)) {
+      $n->removeAttribute('_has_wildcards');
+      $n->removeAttribute('_wildcard'); } }
+  return; }
+
+sub nth_sibling {
+  my ($node, $n) = @_;
+  my $nn = $node;
+  while ($nn && ($n > 1)) { $nn = $nn->nextSibling; $n--; }
+  return $nn; }
+
+sub nth_child {
+  my ($node, $n) = @_;
+  my @c = $node->childNodes;
+  return $c[$n - 1]; }
 #**********************************************************************
 sub compileClause {
   my ($self,   $document, $clause)  = @_;
@@ -190,7 +291,7 @@ sub compileClause {
           map { $self->getLabelID($_) } @$pattern]; }
     else {
       #      $op='select'; $pattern=["descendant-or-self::*[\@label='$pattern']",1]; }}
-      $op = 'select';
+      $op      = 'select';
       $pattern = ["descendant-or-self::*[\@xml:id='" . $self->getLabelID($pattern) . "']", 1]; } }
   elsif ($op eq 'scope') {
     $op = 'select';
@@ -212,7 +313,7 @@ sub compileClause {
     if (ref $pattern eq 'CODE') {
       $op = 'test'; }
     elsif (ref $pattern eq 'ARRAY') {    # Multiple patterns!
-      $op = 'multi_select';
+      $op      = 'multi_select';
       $pattern = [map { $self->compile_match($document, $_) } @$pattern]; }
     else {
       $op = 'select'; $pattern = $self->compile_match($document, $pattern); } }
@@ -247,7 +348,7 @@ sub compile_match1 {
   my ($self, $document, $patternbox) = @_;
   # Create a temporary document
   my $capdocument = LaTeXML::Core::Document->new($document->getModel);
-  my $capture = $capdocument->openElement('_Capture_', font => LaTeXML::Common::Font->new());
+  my $capture     = $capdocument->openElement('_Capture_', font => LaTeXML::Common::Font->new());
   $capdocument->absorb($patternbox);
   my @nodes = ($$self{mode} eq 'math'
     ? $capdocument->findnodes("//ltx:XMath/*", $capture)
@@ -255,16 +356,18 @@ sub compile_match1 {
   my $frag = $capdocument->getDocument->createDocumentFragment;
   map { $frag->appendChild($_) } @nodes;
   # Convert the captured nodes to an XPath that would match them.
-  my $xpath = domToXPath($capdocument, $frag);
+  my ($xpath, $nnodes, @wilds) = domToXPath($capdocument, $frag);
   # The branches of an XMDual can contain "decorations", nodes that are ONLY visible
   # from either presentation or content, but not both.
   # [See LaTeXML::Core::Document->markXMNodeVisibility]
   # These decorations should NOT have rewrite rules applied
   $xpath .= '[@_pvis and @_cvis]' if $$self{math};
 
-  print STDERR "Converting \"" . ToString($patternbox) . "\"\n  => xpath= \"$xpath\"\n"
-    if $LaTeXML::Core::Rewrite::DEBUG;
-  return [$xpath, scalar(@nodes)]; }
+  if ($LaTeXML::Core::Rewrite::DEBUG) {
+    print STDERR "Converting \"" . ToString($patternbox) . "\"\n  $nnodes nodes\n  => xpath= \"$xpath\"\n";
+    foreach my $w (@wilds) {
+      print STDERR 'with wildcard \@' . join(',', @$w) . "\n"; } }
+  return [$xpath, $nnodes, @wilds]; }
 
 # Reworked to do digestion at replacement time.
 sub compile_replacement {
@@ -274,7 +377,6 @@ sub compile_replacement {
     $pattern = $pattern->getBody if $$self{math};
     return sub { $_[0]->absorb($pattern); } }
   else {
-#####    $pattern = Tokenize($$self{math} ? '$' . $pattern . '$' : $pattern) unless ref $pattern;
     return sub {
       my $stomach = $STATE->getStomach;
       $stomach->bgroup;
@@ -304,7 +406,6 @@ sub digest_rewrite {
   $stomach->bgroup;
   $STATE->assignValue(font => LaTeXML::Common::Font->new(), 'local'); # Use empty font, so eventual insertion merges.
   $STATE->assignValue(mathfont => LaTeXML::Common::Font->new(), 'local');
-###  my $box = $stomach->digest((ref $string ? $string : Tokenize($string)), 0);
   my $box = $stomach->digest($string, 0);
   $stomach->egroup;
   return $box; }
@@ -312,34 +413,70 @@ sub digest_rewrite {
 #**********************************************************************
 sub domToXPath {
   my ($document, $node) = @_;
-  return "descendant-or-self::" . domToXPath_rec($document, $node); }
+  my ($xpath, $nnodes, $nwilds, @wilds)
+    = domToXPath_rec($document, $node, 'descendant-or-self', undef);
+  return ($xpath, $nnodes, @wilds); }
 
 # May need some work here;
 my %EXCLUDED_MATCH_ATTRIBUTES = (scriptpos => 1, 'xml:id' => 1, fontsize => 1);    # [CONSTANT]
 
 sub domToXPath_rec {
-  my ($document, $node, @extra_predicates) = @_;
-  my $type = $node->nodeType;
-  if ($type == XML_DOCUMENT_FRAG_NODE) {
-    my @nodes = $node->childNodes;
-    return domToXPath_rec($document, shift(@nodes),
-      domToXPath_seq($document, 'following-sibling', @nodes), @extra_predicates); }
+  my ($document, $node, $axis, $pos) = @_;
+  my $type = (ref $node eq 'XML::LibXML::NodeList' ? 999 : $node->nodeType);
+  if (($type == 999) || ($type == XML_DOCUMENT_FRAG_NODE)) {
+    my @nodes = ($type == 999 ? $node->get_nodelist() : $node->childNodes);
+    my ($xpath, $nnodes, @wilds) = domToXPath_seq($document, $axis, $pos, @nodes);
+    return ($xpath, $nnodes, 0, @wilds); }
   elsif ($type == XML_ELEMENT_NODE) {
-    my $qname = $document->getNodeQName($node);
-    return '*[true()]' if $qname eq '_WildCard_';
+    my $qname      = $document->getNodeQName($node);
+    my @children   = $node->childNodes;
     my @predicates = ();
+    my @wilds      = ();
+    if ($qname eq '_WildCard_') {
+      my $tomatch = $node->childNodes;    # or all children!
+      if ($tomatch) {
+        my ($xpath, $nnodes, $nwilds, @wilds) = domToXPath_rec($document, $tomatch, $axis, $pos);
+        my $n = (scalar(@children) || 1);
+        return ($xpath, $n, $n); }
+      else {
+        return ($axis . '::*', 1, 1); } }
+    # Also, an XMRef pointing to a wildcard is a wildcard!
+    # (or pointing to an XMArg|XMWrap of a wildcard!)
+    elsif ($qname eq 'ltx:XMRef') {
+      my $id = $node->getAttribute('idref');
+      my $r  = $id && $document->lookupID($id);
+      my $rq = $r && $document->getNodeQName($r);    # eq '_WildCard_')
+      if ($rq && ($rq =~ /ltx:(?:XMArg|XMWrap)$/)) {
+        my @rc = $r->childNodes;
+        if ((scalar(@rc) == 1)) {
+          $r  = $r->firstChild;
+          $rq = $document->getNodeQName($r); } }
+      if ($rq && ($rq eq '_WildCard_')) {
+        return ($axis . '::*', 1, 1); } }
+    # Also treat XMArg or XMWrap with single wildcard child as a wildcard (w/o children)
+    elsif (($qname =~ /ltx:(?:XMArg|XMWrap)$/)
+      && (scalar(@children) == 1) && ($document->getNodeQName($children[0]) eq '_WildCard_')) {
+      my $tomatch = $children[0]->childNodes;
+      if ($tomatch) {
+        my ($xpath, $nnodes, $nwilds, @wilds) = domToXPath_rec($document, $tomatch, 'child', 1);
+        return ($axis . '::' . $qname . '[' .
+            join(' and ', ($pos ? ('position()=' . $pos) : undef), $xpath) . ']',
+          1, 1); }
+      else {
+        return ($axis . '::*', 1, 1); } }
     # Order the predicates so as to put most quickly restrictive first.
     if ($node->hasAttributes) {
       foreach my $attribute (grep { $_->nodeType == XML_ATTRIBUTE_NODE } $node->attributes) {
         my $key = $attribute->nodeName;
         next if ($key =~ /^_/) || $EXCLUDED_MATCH_ATTRIBUTES{$key};
         push(@predicates, "\@" . $key . "='" . $attribute->getValue . "'"); } }
-    if ($node->hasChildNodes) {
-      my @children = $node->childNodes;
+    if (@children) {
       if (!grep { $_->nodeType != XML_TEXT_NODE } @children) {    # All are text nodes:
-        push(@predicates, "text()='" . $node->textContent . "'"); }
+        push(@predicates, "text()=" . quoteXPathLiteral($node->textContent)); }
       elsif (!grep { $_->nodeType != XML_ELEMENT_NODE } @children) {
-        push(@predicates, domToXPath_seq($document, 'child', @children)); }
+        my ($xp, $n, @w) = domToXPath_seq($document, 'child', 1, @children);
+        push(@predicates, $xp);
+        push(@wilds,      @w); }
       else {
         Fatal('misdefined', '<rewrite>', $node,
           "Can't generate XPath for mixed content"); } }
@@ -348,20 +485,47 @@ sub domToXPath_rec {
         my $pred = LaTeXML::Common::Font::font_match_xpaths($font);
         push(@predicates, $pred); } }
 
-    return $qname . "[" . join(' and ', grep { $_ } @predicates, @extra_predicates) . "]"; }
+    if ($pos) {
+      unshift(@predicates, 'self::' . $qname);
+      $qname = '*';
+      unshift(@predicates, 'position()=' . $pos); }
+    my $preds = join(' and ', grep { $_ } @predicates);
+    return ($axis . '::' . $qname . ($preds ? "[" . $preds . "]" : ''), 1, 0, @wilds);
+  }
 
   elsif ($type == XML_TEXT_NODE) {
-###    "text()='".$node->textContent."'"; }}
-    return "*[text()='" . $node->textContent . "']"; } }
+    return ("*[text()=" . quoteXPathLiteral($node->textContent) . "]", 1, 0); } }
+
+# Return quoted string, but note: XPath doesn't provide sensible way to slashify ' or "
+sub quoteXPathLiteral {
+  my ($string) = @_;
+  if    ($string !~ /'/) { return "'" . $string . "'"; }
+  elsif ($string !~ /"/) { return '"' . $string . '"'; }
+  else { return 'concat(' . join(',"\'",', map { "'" . $_ . "'"; } split(/'/, $string)) . ')'; } }
 
-# $axis would be child or following-sibling
 sub domToXPath_seq {
-  my ($document, $axis, @nodes) = @_;
-  if (@nodes) {
-    return $axis . "::*[position()=1 and self::"
-      . domToXPath_rec($document, shift(@nodes), domToXPath_seq($document, 'following-sibling', @nodes)) . ']'; }
+  my ($document, $axis, $pos, @nodes) = @_;
+  my $i         = 1;
+  my @sibxpaths = ();
+  my @wilds     = ();
+  my ($xpath, $n, $nwilds, @w0) = domToXPath_rec($document, shift(@nodes), $axis, $pos);
+  if ($nwilds) {
+    for (my $j = 0 ; $j < $nwilds ; $j++) {
+      push(@wilds, [$i]); $i++; } }
   else {
-    return (); } }
+    push(@wilds, (map { [1, @$_]; } @w0));
+    $i++; }
+  foreach my $sib (@nodes) {
+    my ($xp, $nn, $nw, @w) = domToXPath_rec($document, $sib, 'following-sibling', $i - 1);
+    push(@sibxpaths, $xp);
+    if ($nw) {
+      for (my $j = 0 ; $j < $nw ; $j++) {
+        push(@wilds, [$i]); $i++; } }
+    else {
+      push(@wilds, (map { [$i, @$_]; } @w));
+      $i++; } }
+  return ($xpath . (scalar(@sibxpaths) ? join('', map { '[' . $_ . ']'; } @sibxpaths) : ''),
+    $i - 1, @wilds); }
 
 #**********************************************************************
 1;
