@@ -88,36 +88,50 @@ use LaTeXML::Core::Token;    # To get CatCodes
 sub new {
   my ($class, %options) = @_;
   my $self = bless {    # table => {},
-    value => {}, meaning => {}, stash => {}, stash_active => {},
+    value   => {}, meaning  => {}, stash  => {}, stash_active => {},
     catcode => {}, mathcode => {}, sfcode => {}, lccode => {}, uccode => {}, delcode => {},
-    undo => [{ _FRAME_LOCK_ => 1 }], prefixes => {}, status => {},
+    undo    => [{ _FRAME_LOCK_ => 1 }], prefixes => {}, status => {},
     stomach => $options{stomach}, model => $options{model} }, $class;
-  $$self{value}{VERBOSITY} = [0];
+  # Note that "100" is hardwired into TeX, The Program!!!
+  $$self{value}{MAX_ERRORS} = [100];
+  $$self{value}{VERBOSITY}  = [0];
+  # Standard TeX units, in scaled points
+  $$self{value}{UNITS} = [{
+      pt => 65536, pc => 12 * 65536, in => 72.27 * 65536, bp => 72.27 * 65536 / 72,
+      cm => 72.27 * 65536 / 2.54, mm => 72.27 * 65536 / 2.54 / 10, dd => 1238 * 65536 / 1157,
+      cc => 12 * 1238 * 65536 / 1157, sp => 1,
+      px => 72.27 * 65536 / 72,    # Assume px=bp ?
+  }];
+
   $options{catcodes} = 'standard' unless defined $options{catcodes};
   if ($options{catcodes} =~ /^(standard|style)/) {
     # Setup default catcodes.
     my %std = ("\\" => CC_ESCAPE, "{" => CC_BEGIN, "}" => CC_END, "\$" => CC_MATH,
-      "\&" => CC_ALIGN, "\r" => CC_EOL,   "#"  => CC_PARAM, "^" => CC_SUPER,
-      "_"  => CC_SUB,   " "  => CC_SPACE, "\t" => CC_SPACE, "%" => CC_COMMENT,
-      "~" => CC_ACTIVE, chr(0) => CC_IGNORE);
+      "&" => CC_ALIGN,  "\r"   => CC_EOL,    "#"  => CC_PARAM, "^" => CC_SUPER,
+      "_" => CC_SUB,    " "    => CC_SPACE,  "\t" => CC_SPACE, "%" => CC_COMMENT,
+      "~" => CC_ACTIVE, chr(0) => CC_ESCAPE, "\f" => CC_ACTIVE);
     map { $$self{catcode}{$_} = [$std{$_}] } keys %std;
     for (my $c = ord('A') ; $c <= ord('Z') ; $c++) {
       $$self{catcode}{ chr($c) } = [CC_LETTER];
       $$self{catcode}{ chr($c + ord('a') - ord('A')) } = [CC_LETTER]; }
   }
-  $$self{value}{SPECIALS} = [['^', '_', '@', '~', '&', '$', '#', "'"]];
+  $$self{value}{SPECIALS} = [['^', '_', '~', '&', '$', '#', "'"]];
   if ($options{catcodes} eq 'style') {
     $$self{catcode}{'@'} = [CC_LETTER]; }
-  $$self{mathcode} = {};
-  $$self{sfcode}   = {};
-  $$self{lccode}   = {};
-  $$self{uccode}   = {};
-  $$self{delcode}  = {};
+  $$self{mathcode}            = {};
+  $$self{sfcode}              = {};
+  $$self{lccode}              = {};
+  $$self{uccode}              = {};
+  $$self{delcode}             = {};
+  $$self{tracing_definitions} = {};
   return $self; }
 
 sub assign_internal {
   my ($self, $table, $key, $value, $scope) = @_;
   $scope = ($$self{prefixes}{global} ? 'global' : 'local') unless defined $scope;
+  if (exists $$self{tracing_definitions}{$key}) {
+    print STDERR "ASSIGN $key in $table " . ($scope ? "($scope)" : '') . " => " .
+      (ref $value ? $value->stringify : $value) . "\n"; }
   if ($scope eq 'global') {
     # Remove bindings made in all frames down-to & including the next lower locked frame
     my $frame;
@@ -237,7 +251,7 @@ sub valueInFrame {
   $frame = 0 unless defined $frame;
   my $p = 0;
   for (my $f = 0 ; $f < $frame ; $f++) {
-    $p += $$self{undo}[$f]{value}{$key}; }
+    $p += $$self{undo}[$f]{value}{$key} || 0; }
   return $$self{value}{$key}[$p]; }
 
 #======================================================================
@@ -306,27 +320,27 @@ sub assignDelcode {
 #======================================================================
 # Specialized versions of lookup & assign for dealing with definitions
 
-our @active_or_cs = (
+our @CATCODE_ACTIVE_OR_CS = (
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 1, 0, 0,
-  1, 0);
-our @letter_or_other = (
+  1, 0, 0, 0);
+our @CATCODE_LETTER_OR_OTHER = (
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 1,
   1, 0, 0, 0,
-  0, 0);
+  0, 0, 0, 0);
 
 # Get the `Meaning' of a token.  For active control sequence's
 # this may give the definition object (if defined) or another token (if \let) or undef
 # Any other token is returned as is.
 sub lookupMeaning {
   my ($self, $token) = @_;
-  my $e;
   if (my $cs = $token
-    && $active_or_cs[$$token[1]]
+    && $CATCODE_ACTIVE_OR_CS[$$token[1]]
+    && !$$token[2]    # return token itself, if \noexpand
     && $$token[0]) {
     my $e = $$self{meaning}{$cs}; return $e && $$e[0]; }
   else { return $token; } }
@@ -344,13 +358,13 @@ sub assignMeaning {
 # nor cs let to executable tokens
 # This returns a definition object, or undef
 
-# merge of @executable_catcode & @PRIMITIVE_NAME
-our @executable_primitive_name = (    # [CONSTANT]
+# merge of @CATCODE_EXECUTABLE & @CATCODE_PRIMITIVE_NAME
+our @CATCODE_EXECUTABLE_PRIMITIVE_NAME = (    # [CONSTANT]
   undef,       'Begin', 'End', 'Math',
   'Align',     undef,   undef, 'Superscript',
   'Subscript', undef,   undef, undef,
   undef,       undef,   undef, undef,
-  undef,       undef);
+  undef,       undef,   undef, undef);
 
 sub lookupDefinition {
   my ($self, $token) = @_;
@@ -360,9 +374,9 @@ sub lookupDefinition {
   #  my $inmath = $self->lookupValue('IN_MATH');
   my $cc = $$token[1];
   my $lookupname =
-    ($active_or_cs[$cc]
+    ($CATCODE_ACTIVE_OR_CS[$cc]
     ? $$token[0]
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname
     && ($entry = $$self{meaning}{$lookupname})
     && ($defn  = $$entry[0])
@@ -380,9 +394,9 @@ sub lookupConditional {
   #  my $inmath = $self->lookupValue('IN_MATH');
   my $cc = $$token[1];
   my $lookupname =
-    ($active_or_cs[$cc]
+    ($CATCODE_ACTIVE_OR_CS[$cc]
     ? $$token[0]
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname
     && ($entry = $$self{meaning}{$lookupname})
     && ($defn  = $$entry[0])
@@ -400,9 +414,9 @@ sub lookupExpandable {
   #  my $inmath = $self->lookupValue('IN_MATH');
   my $cc = $$token[1];
   my $lookupname =
-    ($active_or_cs[$cc]
+    ($CATCODE_ACTIVE_OR_CS[$cc]
     ? $$token[0]
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname
     && ($entry = $$self{meaning}{$lookupname})
     && ($defn  = $$entry[0])
@@ -427,16 +441,17 @@ sub lookupDigestableDefinition {
   my $cc   = $$token[1];
   my $name = $$token[0];
   my $lookupname =
-    (($active_or_cs[$cc]
-        || ($letter_or_other[$cc] && $self->lookupValue('IN_MATH')
+    (($CATCODE_ACTIVE_OR_CS[$cc]
+        || ($CATCODE_LETTER_OR_OTHER[$cc] && $self->lookupValue('IN_MATH')
         && (($self->lookupMathcode($name) || 0) == 0x8000)))
     ? $name
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname && ($entry = $$self{meaning}{$lookupname})
     && ($defn = $$entry[0])) {
     # If a cs has been let to an executable token, lookup ITS defn.
     if (((ref $defn) eq 'LaTeXML::Core::Token')
-      && ($lookupname = $executable_primitive_name[$$defn[1]])
+      # If we're digesting an unexpanded, act like \relax
+      && ($lookupname = ($$defn[2] ? '\relax' : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$$defn[1]]))
       && ($entry      = $$self{meaning}{$lookupname})) {
       $defn = $$entry[0]; }
     return $defn; }
@@ -449,14 +464,14 @@ sub installDefinition {
   # Ignore attempts to (re)define $cs from tex sources
   #  my $cs = $definition->getCS->getCSName;
   my $token = $definition->getCS;
-  my $cs = ($LaTeXML::Core::Token::PRIMITIVE_NAME[$$token[1]] || $$token[0]);
+  my $cs    = ($LaTeXML::Core::Token::CATCODE_PRIMITIVE_NAME[$$token[1]] || $$token[0]);
   if ($self->lookupValue("$cs:locked") && !$LaTeXML::Core::State::UNLOCKED) {
-    if (my $s = $self->getStomach->getGullet->getSource) {
-      # report if the redefinition seems to come from document source
-      if ((($s eq "Anonymous String") || ($s =~ /\.(tex|bib)$/))
-        && ($s !~ /\.code\.tex$/)) {
-        Info('ignore', $cs, $self->getStomach, "Ignoring redefinition of $cs"); }
-      return; } }
+    my $s = $self->getStomach->getGullet->getSource;
+    # report if the redefinition seems to come from document source
+    if (((!defined($s)) || ($s =~ /\.(tex|bib)$/))
+      && ($s !~ /\.code\.tex$/)) {
+      Info('ignore', $cs, $self->getStomach, "Ignoring redefinition of $cs"); }
+    return; }
   assign_internal($self, 'meaning', $cs => $definition, $scope);
   return; }
 
@@ -471,6 +486,32 @@ sub installDefinition {
 
 #======================================================================
 
+# Generate a stub definition for an undefined control-sequence,
+# along with appropriate error messge.
+sub generateErrorStub {
+  my ($self, $caller, $token, $params) = @_;
+  my $cs = $token->getCSName;
+  $self->noteStatus(undefined => $cs);
+  # To minimize chatter, go ahead and define it...
+  if ($cs =~ /^\\if(.*)$/) {    # Apparently an \ifsomething ???
+    my $name = $1;
+    Error('undefined', $token, $caller, "The token " . $token->stringify . " is not defined.",
+      "Defining it now as with \\newif");
+    $self->installDefinition(LaTeXML::Core::Definition::Expandable->new(
+        T_CS('\\' . $name . 'true'), undef, '\let' . $cs . '\iftrue'));
+    $self->installDefinition(LaTeXML::Core::Definition::Expandable->new(
+        T_CS('\\' . $name . 'false'), undef, '\let' . $cs . '\iffalse'));
+    LaTeXML::Package::Let($token, T_CS('\iffalse')); }
+  else {
+    Error('undefined', $token, $caller, "The token " . $token->stringify . " is not defined.",
+      "Defining it now as <ltx:ERROR/>");
+    $self->installDefinition(LaTeXML::Core::Definition::Constructor->new($token, $params,
+        sub { $_[0]->makeError('undefined', $cs); }),
+      'global'); }
+  return $token; }
+
+#======================================================================
+
 sub pushFrame {
   my ($self, $nobox) = @_;
   # Easy: just push a new undo hash.
@@ -535,7 +576,7 @@ sub pushDaemonFrame {
         my $type  = ref $value;
         if (($type eq 'HASH') || ($type eq 'ARRAY')) {    # Only concerned with mutable perl data?
                                                           # Local assignment
-          $$frame{$table}{$key} = 1;                      # Note new value in this frame.
+          $$frame{$table}{$key} = 1;                                  # Note new value in this frame.
           unshift(@{ $$hash{$key} }, daemon_copy($value)); } } } }    # And push new binding.
       # Record the contents of LaTeXML::Package::Pool as preloaded
   my $pool_preloaded_hash = { map { $_ => 1 } keys %LaTeXML::Package::Pool:: };
@@ -568,7 +609,7 @@ sub popDaemonFrame {
       unless (exists $$pool_preloaded_hash{$subname}) {
         undef $LaTeXML::Package::Pool::{$subname};
         delete $LaTeXML::Package::Pool::{$subname};
-      } }
+    } }
     # Finally, pop the frame
     $self->popFrame; }
   else {
@@ -605,7 +646,7 @@ sub activateScope {
         # Here we ALWAYS push the stashed values into the table
         # since they may be popped off by deactivateScope
         my ($table, $key, $value) = @$entry;
-        $$frame{$table}{$key}++;    # Note that this many values must be undone
+        $$frame{$table}{$key}++;                             # Note that this many values must be undone
         unshift(@{ $$self{$table}{$key} }, $value); } } }    # And push new binding.
   return; }
 
@@ -643,26 +684,21 @@ sub getActiveScopes {
 
 #======================================================================
 # Units.
-#   Put here since it could concievably evolve to depend on the current font.
-
-# Conversion to scaled points
-my %UNITS = (    # [CONSTANT]
-  pt => 65536, pc => 12 * 65536, in => 72.27 * 65536, bp => 72.27 * 65536 / 72,
-  cm => 72.27 * 65536 / 2.54, mm => 72.27 * 65536 / 2.54 / 10, dd => 1238 * 65536 / 1157,
-  cc => 12 * 1238 * 65536 / 1157, sp => 1);
 
 sub convertUnit {
   my ($self, $unit) = @_;
   $unit = lc($unit);
+  # Put here since it could concievably evolve to depend on the current font.
   # Eventually try to track font size?
   if    ($unit eq 'em') { return 10.0 * 65536; }
   elsif ($unit eq 'ex') { return 4.3 * 65536; }
   elsif ($unit eq 'mu') { return 10.0 * 65536 / 18; }
   else {
-    my $sp = $UNITS{$unit};
+    my $units = $self->lookupValue('UNITS');
+    my $sp    = $$units{$unit};
     if (!$sp) {
       Warn('expected', '<unit>', undef, "Illegal unit of measure '$unit', assuming pt.");
-      $sp = $UNITS{'pt'}; }
+      $sp = $$units{'pt'}; }
     return $sp; } }
 
 #======================================================================
@@ -720,7 +756,7 @@ sub getStatusCode {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
