@@ -56,7 +56,7 @@ sub ProcessChain_internal {
     foreach my $doc (@docs) {
       local $LaTeXML::Post::DOCUMENT = $doc;
       if (my @nodes = grep { $_ } $processor->toProcess($doc)) {    # If there are nodes to process
-        my $n = scalar(@nodes);
+        my $n   = scalar(@nodes);
         my $msg = join(' ', $processor->getName || '',
           $doc->siteRelativeDestination || '',
           ($n > 1 ? "$n to process" : 'processing'));
@@ -106,7 +106,7 @@ sub getStatusMessage {
 sub getsorter {
   my ($self, $lang) = @_;
   my $collator;
-  if ($collator = $$self{collatorcache}{$lang}) { }
+  if    ($collator = $$self{collatorcache}{$lang}) { }
   elsif ($collator = eval {
       local $LaTeXML::IGNORE_ERRORS = 1;
       require 'Unicode/Collate/Locale.pm';
@@ -195,16 +195,21 @@ sub desiredResourcePathname {
 # but I think we've accommodated absolute ones.
 sub generateResourcePathname {
   my ($self, $doc, $node, $source, $type) = @_;
-  my $subdir = $$self{resource_directory} || '';
-  my $prefix = $$self{resource_prefix}    || "x";
+  my $subdir  = $$self{resource_directory} || '';
+  my $prefix  = $$self{resource_prefix}    || "x";
   my $counter = join('_', "_max", $subdir, $prefix, "counter_");
-  my $n = $doc->cacheLookup($counter) || 0;
-  my $name = $prefix . ++$n;
+  my $n       = $doc->cacheLookup($counter) || 0;
+  my $name    = $prefix . ++$n;
   $doc->cacheStore($counter, $n);
   return pathname_make(dir => $subdir, name => $name, type => $type); }
 
-# Get a list [class,classoptions, oldstyle],[package,packageoptions],...]
-# The options are strings
+# Returns a two-part list of the form:
+#
+# [class, classoptions, oldstyle], [package1,package1options], [package2,package2options], ...
+#
+# Where there first element is always the *class* (if none, "article" returned as a default)
+# And all following elements are *packages*
+#
 sub find_documentclass_and_packages {
   my ($self, $doc) = @_;
   my ($class, $classoptions, $oldstyle, @packages);
@@ -218,13 +223,24 @@ sub find_documentclass_and_packages {
       $classoptions = $$entry{options} || 'onecolumn';
       $oldstyle     = $$entry{oldstyle}; }
     elsif ($$entry{package}) {
-      push(@packages, [$$entry{package}, $$entry{options} || '']); }
-  }
+      my @p = grep { $_; } split(/\s*,\s*/, $$entry{package});
+      foreach my $package (@p) {
+        push(@packages, [$package, $$entry{options} || '']); } } }
   if (!$class) {
     Warn('expected', 'class', undef, "No document class found; using article");
     $class = 'article'; }
   return ([$class, $classoptions, $oldstyle], @packages); }
 
+sub find_preambles {
+  my ($self, $doc) = @_;
+  my @preambles = ();
+  foreach my $pi ($doc->findnodes(".//processing-instruction('latexml')")) {
+    my $data = $pi->textContent;
+    while ($data =~ s/\s*([\w\-\_]*)=([\"\'])(.*?)\2//) {
+      if ($1 eq 'preamble') {
+        push(@preambles, $3); } } }
+  return join("\n", @preambles); }
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 package LaTeXML::Post::MathProcessor;
 use strict;
@@ -336,7 +352,7 @@ sub processNode {
   # XMath will be removed (LATER!), but mark its ids as reusable.
   $doc->preremoveNodes($xmath);
   if ($$self{parallel}) {
-    my $primary = $self->convertNode($doc, $xmath);
+    my $primary     = $self->convertNode($doc, $xmath);
     my @secondaries = ();
     foreach my $proc (@{ $$self{secondary_processors} }) {
       local $LaTeXML::Post::MATHPROCESSOR = $proc;
@@ -425,7 +441,7 @@ sub convertXMTextContent {
       my $tag = $doc->getQName($node);
       if ($tag eq 'ltx:XMath') {
         my $conversion = $self->convertNode($doc, $node);
-        my $xml = $$conversion{xml};
+        my $xml        = $$conversion{xml};
         # And if no xml ????
         push(@result, $self->outerWrapper($doc, $node, $xml)); }
       else {
@@ -475,6 +491,7 @@ sub associateNode {
   # What kind of branch are we generating?
   my $ispresentation = $self->rawIDSuffix eq '.pmml';    # TEMPORARY HACK: BAAAAD method!
   my $iscontainer    = 0;
+  my $container;
   if ($isarray) {
     return if $$node[1]{'_sourced'};
     $$node[1]{'_sourced'} = 1;
@@ -485,28 +502,38 @@ sub associateNode {
     $node->setAttribute('_sourced' => 1);
     $iscontainer = scalar(element_nodes($node)); }
   my $sourcenode = $currentnode;
+  # If the current node is declared, use it (but meaning is overridden)
+  if ($currentnode->getAttribute('decl_id')) { }
   # If the generated node is a "container" (non-token!), use the containing XMDual as source
-  if ($iscontainer) {
+  elsif ($iscontainer) {
     my $sid = $sourcenode->getAttribute('xml:id');
     # But ONLY if that XMDual is the "direct" parent, or is parent of XRef that points to $current
-    if (my $container = $document->findnode('parent::ltx:XMDual[1]', $sourcenode)
+    if ($container = $document->findnode('parent::ltx:XMDual[1]', $sourcenode)
       || ($sid &&
         $document->findnode("ancestor-or-self::ltx:XMDual[ltx:XMRef[\@idref='$sid']][1]",
           $sourcenode))) {
       $sourcenode = $container; } }
+  # Parent App w/decl_id or meaning is source, unless current node has decl_id (but ignore meaning)
+  elsif ($container = $document->findnode('ancestor::ltx:XMApp[@decl_id or @meaning][1]', $sourcenode)) {
+    $sourcenode = $container; }
   # If the current node is appropriately visible, use it.
   elsif ($currentnode->getAttribute(($ispresentation ? '_cvis' : '_pvis'))) { }
   # Else (current node isn't visible); try to find content OPERATOR
-  elsif (my $container = $document->findnode('ancestor-or-self::ltx:XMDual[1]', $sourcenode)) {
+  elsif ($container = $document->findnode('ancestor-or-self::ltx:XMDual[1]', $sourcenode)) {
     my ($op) = element_nodes($container);
     my $q = $document->getQName($op) || 'unknown';
-    if ($q eq 'ltx:XMTok') { }
+    if ($container->hasAttribute('decl_id')) {
+      $op = undef; }
+    elsif ($q eq 'ltx:XMTok') { }
     elsif ($q eq 'ltx:XMApp') {
-      ($op) = element_nodes($op);
-      $q = $document->getQName($op) || 'unknown'; }    # get "real" operator
+      while (($q eq 'ltx:XMApp') && !$op->hasAttribute('meaning')) {
+        ($op) = element_nodes($op);
+        $q = $document->getQName($op) || 'unknown'; } }    # get "real" operator
     if ($q eq 'ltx:XMRef') {
       $op = $document->realizeXMNode($op); }
-    if ($op && !$op->getAttribute('_pvis')) {
+    # Be a bit fuzzy about whether something is "visible"
+    if ($op && !($op->getAttribute('_pvis')
+        && (($op->getAttribute('thickness') || '<anything>') ne '0pt'))) {
       $sourcenode = $op; }
     else {
       $sourcenode = $container; } }
@@ -516,7 +543,7 @@ sub associateNode {
       $document->generateNodeID($sourcenode, '', 1); }         # but the ID is reusable
     if (my $sourceid = $sourcenode->getAttribute('fragid')) {    # If source has ID
       my $nodeid = $currentnode->getAttribute('fragid') || $sourceid;
-      my $id = $document->uniquifyID($nodeid, $self->IDSuffix);
+      my $id     = $document->uniquifyID($nodeid, $self->IDSuffix);
       if ($isarray) {
         $$node[1]{'xml:id'} = $id; }
       else {
@@ -645,9 +672,9 @@ sub new_internal {
     else {
       $data{siteDirectory} = $data{destinationDirectory}; } }
   # Start, at least, with our own namespaces.
-  $data{namespaces}    = { ltx    => $NSURI } unless $data{namespaces};
-  $data{namespaceURIs} = { $NSURI => 'ltx' }  unless $data{namespaceURIs};
-  $data{idcache}       = {};
+  $data{namespaces}       = { ltx    => $NSURI } unless $data{namespaces};
+  $data{namespaceURIs}    = { $NSURI => 'ltx' }  unless $data{namespaceURIs};
+  $data{idcache}          = {};
   $data{idcache_reusable} = {};
   $data{idcache_reserve}  = {};
 
@@ -737,11 +764,13 @@ sub setDocument_internal {
     foreach my $node ($self->findnodes("//*[\@xml:id]")) {    # Now record all ID's
       $$self{idcache}{ $node->getAttribute('xml:id') } = $node; }
     # Fetch any additional namespaces from the root
-    foreach my $ns ($root->documentElement->getNamespaces) {
-      my ($prefix, $uri) = ($ns->getLocalName, $ns->getData);
-      if ($prefix) {
-        $$self{namespaces}{$prefix} = $uri    unless $$self{namespaces}{$prefix};
-        $$self{namespaceURIs}{$uri} = $prefix unless $$self{namespaceURIs}{$uri}; } }
+
+    if (my $docroot = $root->documentElement) {
+      foreach my $ns ($docroot->getNamespaces) {
+        my ($prefix, $uri) = ($ns->getLocalName, $ns->getData);
+        if ($prefix) {
+          $$self{namespaces}{$prefix} = $uri    unless $$self{namespaces}{$prefix};
+          $$self{namespaceURIs}{$uri} = $prefix unless $$self{namespaceURIs}{$uri}; } } }
 
     # Extract data from latexml's ProcessingInstructions
     # I'd like to provide structured access to the PI's for those modules that need them,
@@ -754,7 +783,9 @@ sub setDocument_internal {
     foreach my $pi (@{ $$self{processingInstructions} }) {
       if ($pi =~ /^\s*searchpaths\s*=\s*([\"\'])(.*?)\1\s*$/) {
         push(@paths, split(',', $2)); } }
-    push(@paths, pathname_absolute($$self{sourceDirectory})) if $$self{sourceDirectory};
+    ### No, this ultimately can be the xml source, which may be the destination;
+    ### adding this gets the wrong graphics (already processed!)
+    ### push(@paths, pathname_absolute($$self{sourceDirectory})) if $$self{sourceDirectory};
     $$self{searchpaths} = [@paths]; }
   elsif ($roottype eq 'XML::LibXML::Element') {
     $$self{document} = XML::LibXML::Document->new("1.0", "UTF-8");
@@ -775,7 +806,7 @@ sub setDocument_internal {
     my ($tag, $attributes, @children) = @$root;
     my ($prefix, $localname) = $tag =~ /^(.*):(.*)$/;
     my $nsuri = $$self{namespaces}{$prefix};
-    my $node = $$self{document}->createElementNS($nsuri, $localname);
+    my $node  = $$self{document}->createElementNS($nsuri, $localname);
     $$self{document}->setDocumentElement($node);
     map { $$attributes{$_} && $node->setAttribute($_ => $$attributes{$_}) } keys %$attributes
       if $attributes;
@@ -901,7 +932,9 @@ sub validate {
       $rng->validate($$self{document}); };
     LaTeXML::Post::Error("malformed", 'document', undef,
       "Document fails RelaxNG validation (" . $schema . ")",
-      "Validation reports: " . $@) if $@ || !defined $v; }
+      "Validation reports: " . $@,
+      "(Jing may provide a more precise report; https://relaxng.org/jclark/jing.html)")
+      if $@ || !defined $v; }
   elsif (my $decldtd = $$self{document}->internalSubset) {    # Else look for DTD Declaration
     my $dtd = XML::LibXML::Dtd->new($decldtd->publicId, $decldtd->systemId);
     if (!$dtd) {
@@ -927,7 +960,7 @@ sub idcheck {
   my %missing = ();
   foreach my $node ($self->findnodes("//*[\@xml:id]")) {
     my $id = $node->getAttribute('xml:id');
-    $dups{$id} = 1 if $idcache{$id};
+    $dups{$id}    = 1 if $idcache{$id};
     $idcache{$id} = 1; }
   foreach my $id (keys %{ $$self{idcache} }) {
     $missing{$id} = 1 unless $idcache{$id}; }
@@ -1068,7 +1101,7 @@ sub addNodes {
           my $atype = $attr->nodeType;
           if ($atype == XML_ATTRIBUTE_NODE) {
             my $key = $attr->nodeName;
-            if ($key =~ /^_/) { }    # don't copy internal attributes
+            if    ($key =~ /^_/) { }     # don't copy internal attributes
             elsif ($key eq 'xml:id') {
               my $value = $attr->getValue;
               my $old;
@@ -1103,7 +1136,7 @@ sub removeNodes {
   my ($self, @nodes) = @_;
   foreach my $node (@nodes) {
     my $ref = ref $node;
-    if (!$ref) { }
+    if    (!$ref) { }
     elsif ($ref eq 'ARRAY') {
       my ($t, $a, @n) = @$node;
       if (my $id = $$a{'xml:id'}) {
@@ -1125,7 +1158,7 @@ sub preremoveNodes {
   my ($self, @nodes) = @_;
   foreach my $node (@nodes) {
     my $ref = ref $node;
-    if (!$ref) { }
+    if    (!$ref) { }
     elsif ($ref eq 'ARRAY') {
       my ($t, $a, @n) = @$node;
       if (my $id = $$a{'xml:id'}) {
@@ -1181,13 +1214,14 @@ sub prependNodes {
 sub cloneNode {
   my ($self, $node, $idsuffix, %options) = @_;
   return $node unless ref $node;
+  return $node if ref $node eq 'ARRAY'; # Should we deep clone if we get an array? Just return for now
   my $copy    = $node->cloneNode(1);
   my $nocache = $options{nocache};
 ####  $idsuffix = '' unless defined $idsuffix;
   # Find all id's defined in the copy and change the id.
   my %idmap = ();
   foreach my $n ($self->findnodes('descendant-or-self::*[@xml:id]', $copy)) {
-    my $id = $n->getAttribute('xml:id');
+    my $id    = $n->getAttribute('xml:id');
     my $newid = $self->uniquifyID($id, $idsuffix);
     $idmap{$id} = $newid;
     $self->recordID($newid => $n) unless $nocache;
@@ -1364,7 +1398,7 @@ sub realizeXMNode {
         if (my $realnode = $self->findNodeByID($id)) {
           $node = $realnode; }
         else {
-          Fatal('expected', 'id', undef, "Cannot find a node with xml:id='$id'");
+          Error('expected', 'id', undef, "Cannot find a node with xml:id='$id'");
           return; } }
       elsif ($qname eq 'ltx:XMDual') {
         my ($content, $presentation) = element_nodes($node);
@@ -1376,7 +1410,7 @@ sub realizeXMNode {
     if (my $realnode = $self->findNodeByID($id)) {
       return $realnode; }
     else {
-      Fatal('expected', 'id', undef, "Cannot find a node with xml:id='$id'");
+      Error('expected', 'id', undef, "Cannot find a node with xml:id='$id'");
       return; } }
   return $node; }
 
